/*
 * ProcedurySHT3x.c
 *
 *  Created on: 25.09.2020
 *      Author: ryszard
 */

#include "ProcedurySHT3.h"

#include "Procedury_I2C_hard.h"

char CRC8(const char *data,int length)
{
    uint8_t crc = 0xff;
    size_t i, j;
    for (i = 0; i < length; i++) {
        crc ^= data[i];
        for (j = 0; j < 8; j++) {
            if ((crc & 0x80) != 0)
                crc = (uint8_t)((crc << 1) ^ 0x31);
            else
                crc <<= 1;
        }
    }
    return crc;
}

//------------------------
//odczyt rejestru statusu
uint8_t OdczytRejestruStatusu(uint8_t *status)
//we: status wskaźnik do buforu 2 bajtów odczytanego statusu + 1 bajtu CRC
//wy: TRUE sukces
{
#define READ_OUT_STATUS_REG	0xF32D
uint8_t sukces;
unsigned char slave_adr, ile_bajtow_danych;
char bufor[2];

	slave_adr =SHT3I2CADDRL_7B<<1;
	ile_bajtow_danych =2;
	bufor[0] =READ_OUT_STATUS_REG>>8;
	bufor[1] =0xff & READ_OUT_STATUS_REG;

	sukces =I2C_ZapisBloku(slave_adr, MEM_ADRES_I2C_0BAJT,
		0, ile_bajtow_danych, bufor);
	if (sukces ==FALSE) return FALSE;
	ile_bajtow_danych =3;
	sukces =I2C_OdczytBloku(slave_adr, MEM_ADRES_I2C_0BAJT,
		0, ile_bajtow_danych, (char *)status);
	if (sukces ==FALSE) return FALSE;

	char crc8 =CRC8((char *)status,2);
	char crc8_odczyt =*(status+2);
	if (crc8 !=crc8_odczyt) return FALSE;

	return TRUE;
}
//------------------------
//odczyt rejestrów temperatury i wilgotności
uint8_t OdczytTemperaturyIWilgotnosci_Single(uint8_t *p_bufor, double *temperatura, double *wilgotnosc)
{
#define REPEAT_MED_CLOCKSTRETCH_DIS	0x240B
uint8_t sukces;
unsigned char slave_adr, ile_bajtow_danych;
char bufor[6];

	slave_adr =SHT3I2CADDRL_7B<<1;
	ile_bajtow_danych =2;
	bufor[0] =REPEAT_MED_CLOCKSTRETCH_DIS>>8;
	bufor[1] =0xff & REPEAT_MED_CLOCKSTRETCH_DIS;

	sukces =I2C_ZapisBloku(slave_adr, MEM_ADRES_I2C_0BAJT,
		0, ile_bajtow_danych, bufor);
	if (sukces ==FALSE) return FALSE;

	ile_bajtow_danych =6;
	HAL_Delay(20);	//czas konwersji

	sukces =I2C_OdczytBloku(slave_adr, MEM_ADRES_I2C_0BAJT,
		0, ile_bajtow_danych, bufor);
	if (sukces ==FALSE) return FALSE;

	char crc8 =CRC8((char *)bufor,2);
	char crc8_odczyt =*(bufor+2);
	if (crc8 !=crc8_odczyt) return FALSE;

	crc8 =CRC8((char *)(bufor+3),2);
	crc8_odczyt =*(bufor+5);
	if (crc8 !=crc8_odczyt) return FALSE;
	*wilgotnosc =KonwersjaSurowychDanychnaWilgotnosc((uint8_t *)(bufor+3));

	*p_bufor =bufor[0];
	*(p_bufor+1) =bufor[1];
	*(p_bufor+2) =bufor[3];
	*(p_bufor+3) =bufor[4];

	*temperatura =KonwersjaSurowychDanychnaTemperature((uint8_t *)bufor);

	return TRUE;
}
//------------------------
//konwersja wartości odczytanych rejestrów na temperaturę
double KonwersjaSurowychDanychnaTemperature(uint8_t *p_buf)
{
	uint16_t reg_raw =*p_buf <<8;
	reg_raw =reg_raw + *(p_buf+1);
	double temperatura =reg_raw;
	temperatura =temperatura / (65536-1);
	temperatura =temperatura * 175;
	temperatura =temperatura + (-45);
	return 	temperatura;

}
//------------------------
//konwersja wartości odczytanych rejestrów na wilgotność
double KonwersjaSurowychDanychnaWilgotnosc(uint8_t *p_buf)
{
	uint16_t reg_raw =*p_buf <<8;
	reg_raw =reg_raw + *(p_buf+1);
	double wilgotnosc =reg_raw;
	wilgotnosc =wilgotnosc / (65536-1);
	wilgotnosc =wilgotnosc * 100;
	return wilgotnosc;
}
//------------------------
//podgrzewanie włąćż/wyłącz
uint8_t Podgrzewanie_OnOff(uint8_t podgrzewanie_wlacz)
{
#define HEATER_ENABLE	0x306d
#define HEATER_DISABLE	0x3066

uint8_t sukces;
unsigned char slave_adr, ile_bajtow_danych;
char bufor[6];

	slave_adr =SHT3I2CADDRL_7B<<1;
	ile_bajtow_danych =2;
	if (podgrzewanie_wlacz==TRUE)
	{
		bufor[0] =HEATER_ENABLE>>8;
		bufor[1] =0xff & HEATER_ENABLE;
	}
	else
	{
		bufor[0] =HEATER_DISABLE>>8;
		bufor[1] =0xff & HEATER_DISABLE;
	}
	sukces =I2C_ZapisBloku(slave_adr, MEM_ADRES_I2C_0BAJT,
		0, ile_bajtow_danych, bufor);
	if (sukces ==FALSE) return FALSE;

	return TRUE;
}
